/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using com.trackplus.webservice.client.trackplusWS;
using log4net;
using log4net.Config;

namespace com.trackplus.track.ws.tcl
{
    /**
 * This class contains the entire interface to the Track+ web service
 * on the client side. The version of this class has to be compatible to the version
 * on the Track+ server. The corresponding class on the server side is
 * {@link TSLFacade}.
 *
 * @author Adrian Bojani
 * @version 5.0.4
 *
 */
    public class TCLFacade
    {
        private static readonly ILog LOGGER = LogManager.GetLogger(typeof(TCLFacade));

        public const String STATUS_OK = "0";
        /**
         * This status is returned when a data-base exception occurs.
         */
        public const String STATUS_DB_EXCEPTION = "1";
        /**
         * This status is returned when a business logic exception is thrown.
         */
        public const String STATUS_LOGIC_EXCEPTION = "2";
        /**
         * This status is returned when some data are not valid
         */
        public const String STATUS_VALIDATION_EXCEPTION = "3";
        /**
         * ISO date time format used to send date-time over web-service
         */
        public const String ISO_DATE_TIME_FORMAT = "yyyy-MM-dd H:mm:ss.S";
        /**
         * ISO date format used to send date over web-service
         */
        public const String ISO_DATE_FORMAT = "yyyy-MM-dd";

        /**
         * Vertical alignment
         */
        public const int TOP = 0;

        /**
         * Vertical alignment
         */
        public const int BOTTOM = 1;

        /**
         * Vertical alignment
         */
        public const int MIDDLE = 2;

        /**
         * Horizontal alignment
         */
        public const int LEFT = 3;

        /**
         * Horizontal alignment
         */
        public const int RIGHT = 4;

        /**
         * Horizontal alignment
         */
        public const int CENTER = 5;

        public const String clientVersion = "5.0.4";


        private TrackplusServicePortTypeClient trackWebService = null;

        private WSLogonBean logonBean;
        //timeout 1 min
        public const long TIME_OUT_MILISECONDS = 60000;

        /**
     * The constructor. Before using any of the other methods in this class, you need to create a connector
     * to the Track+ server using this constructor.
     * @param endpoint like <code>http://www.yourdomain.com/track/services/TrackplusService</code>
     * @throws RemoteException
     */
        public TCLFacade(String endpoint) /*throws RemoteException,TCLFacadeException*/{
            this.initTCLFacade(endpoint, null, null);
        }

        /**
         * The constructor. Before using any of the other methods in this class, you need to create a connector
         * to the Track+ server using this constructor.
         * @param endpoint like <code>http://www.yourdomain.com/track/services/TrackplusService</code>
         * @param cbaUser in case of container based authentication the user name
         * @param cbaPassword in case of container based authentication the user password
         * @throws RemoteException
         */
        public TCLFacade(String endpoint, String cbaUser, String cbaPassword) /*throws RemoteException,TCLFacadeException*/{
            this.initTCLFacade(endpoint, cbaUser, cbaPassword);
        }

        /**
         * The constructor. Before using any of the other methods in this class, you need to create a connector
         * to the Track+ server using this constructor.
         * @param endpoint like <code>http://www.yourdomain.com/track/services/TrackplusService</code>
         * @param cbaUser in case of container based authentication the user name
         * @param cbaPassword in case of container based authentication the user password
         * @throws RemoteException
         */
        private void initTCLFacade(String endpoint, String cbaUser, String cbaPassword) /*throws RemoteException,TCLFacadeException*/{
            //LOGGER.Debug("Trying to create TSLFacadeStub for endpoint:"+endpoint);

            String url;
            if (endpoint == null || endpoint.Length == 0)
            {
                throw new TCLFacadeException("Can't create TCLFade with empty endpoint");
            }
            if (endpoint.EndsWith("/services/TrackplusService"))
            {
                url = endpoint;
            }
            else
            {
                if (endpoint.EndsWith("/"))
                {
                    url = endpoint + "services/TrackplusService";
                }
                else
                {
                    url = endpoint + "/services/TrackplusService";
                }
            }

            if (url.StartsWith("https"))
            {
                trackWebService = new TrackplusServicePortTypeClient("TrackplusServiceHttpsSoap11Endpoint", url);
            }
            else
            {
                trackWebService = new TrackplusServicePortTypeClient("TrackplusServiceHttpSoap11Endpoint", url);
            }

            try
            {
                //test the webservice
                trackWebService.getServerVersionAndCompatible();
            }
            catch (Exception ex)
            {
                LOGGER.Debug(ex);
                throw new TCLFacadeException("Invalid url: " + ex);
            }
        }

        /**
     * The constructor. Before using any of the other methods in this class, you need to create a connector
     * to the Track+ server using this constructor.
     * @param endpoint like <code>http://www.yourdomain.com/track/services/TrackplusService</code>
     * @param cbaUser the container base authentication user name
     * @param cbaPassword the container base authentication user password
     * @throws RemoteException
     * @throws TCLFacadeException
     */
        // public TCLFacade(String endpoint,String cbaUser,String cbaPassword) /* throws RemoteException,TCLFacadeException */{
        // 	this(endpoint,null,0,null,null,cbaUser,cbaPassword);
        // }


        /**
         * This method is used to log in the client with the user specific data
         * stored in the {@link WSLogonBean} (mainly user name and password). The
         * return value is a {@link WSResponsePerson} which provides all user specific
         * data, such as last name, first name, preferred locale and so on.
         * 
         * <h4>Example</h4> 
         * {@link com.trackplus.track.ws.samples.LoginSample}

         * <p></p>
         * @param logonBean contains user name and password
         * @return a person bean
         * @throws RemoteException in case of connection problems
         * @throws TCLFacadeException if on the server are business logic exceptions
         */
        public WSPersonBean login(WSLogonBean logonBean) /*throws RemoteException,TCLFacadeException*/{
            if (!checkVersion())
            {
                throw new TCLFacadeException("Incompatible versions!");
            }
            WSRequestLogon requestLogon = new WSRequestLogon();
            requestLogon.logonBean = logonBean;

            WSResponsePerson reply = trackWebService.login(requestLogon);

            if (reply.person == null)
            {
                LOGGER.Debug("Login failed!");
                LOGGER.Info(reply.errorMessages[0]);
                return null;
            }
            LOGGER.Debug("Login reply status=" + reply.status);
            checkResponse(reply);
            this.logonBean = logonBean;
            LOGGER.Debug("Login successful!");
            return reply.person;
        }

        /**
         * This method is used to log off the user from the Track+ server
         *
         * <h4>Example</h4> 
         * {@link com.trackplus.track.ws.samples.LoginSample}
         * <p></p> 
         * @return bool 
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public bool logoff() /*throws RemoteException,TCLFacadeException*/{
            WSResponseBoolean reply = trackWebService.logoff();
            checkResponse(reply);
            LOGGER.Debug("logoff");
            return reply.value;
        }

        /**
         * This method returns the current login status.
         * 
         * @return true if the user is logged on, false if it is not.
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public bool isLoggedIn() /*throws RemoteException,TCLFacadeException*/{
            WSResponseBoolean reply = trackWebService.isLoggedIn();
            checkResponse(reply);
            return reply.value;
        }

        /**
         * Reconnect with the server. Takes the existing {@link WSLogonBean} for user name
         * and password.
         * @return true if a reconnect was required, false if the connection was already established.
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public bool reconnect() /*throws RemoteException,TCLFacadeException*/ {
            if (this.logonBean != null)
            {
                login(logonBean);
                return true;
            }
            else
            {
                return false;
            }
        }

        /**
         * It is assumed that the interface evolves over time. Therefore we need to
         * make sure that client and server interface versions are compatible. For this
         * purpose the server holds a list with client versions that it supports.
         * This method serves to check if the interfaces match.
         * 
         * <h4>Example</h4> 
         * {@link com.trackplus.track.ws.samples.LoginSample}
         * <p></p>
         *  
         * @return true if version is compatible
         * @throws RemoteException 
         * @throws TCLFacadeException
         * 
         */
        public bool checkVersion() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Check version...");

            WSRequestString requestString = new WSRequestString();
            requestString.value = clientVersion;

            WSResponseBoolean reply = trackWebService.checkVersion(requestString);

            checkResponse(reply);
            return reply.value;
        }

        /**
         * Get the server version and compatible client versions. 
         * String[0] is the server version, and the remaining elements in the array are the
         * compatible client versions.
         * 
         * <h4>Example</h4> 
         * {@link com.trackplus.track.ws.samples.LoginSample}
         * <p></p>
         * 
         * @return a String[] array with versions
         * @throws RemoteException 
         * @throws TCLFacadeException 
         * 
         */
        public String[] getServerVersionAndCompatible() /*throws RemoteException,TCLFacadeException*/{
            WSResponseStringArray reply = trackWebService.getServerVersionAndCompatible();
            checkResponse(reply);
            return reply.value;
        }

        /**
         * Get the version of this client interface.
         * @return a String with the version of this client interface
         */
        public String getInterFaceVersion()
        {
            return clientVersion;
        }


        //-------------------END OF Authentication handling ----------------------------	


        //------------------- Query handling -------------------------------------------	

        /**
         * Get the nodes for a filter- (item or automail) or for reports-picker.
         * <p>
         * The format of the nodeIDs for folders (projects and categories) are the same as for 
         * the non picker version but the leafs are integers.
         * <p>
         * A picker will return an integer (instead of a complicated nodeID, but for
         * folders the IDs can't be integers because that would cause nodes with possibly 
         * duplicated IDs). 
         * 
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.QuerySample}
         * <p></p>
         * 
         * @return {@link WSTreeNode} a tree with folders  and/or queries.
         * @throws RemoteException 
         * @throws TCLFacadeException 
         * 
         */
        public WSTreeNode[] getQueries() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get queries");

            WSResponseTreeNodeArray response = trackWebService.getQueries();
            checkResponse(response);
            return response.nodes;
        }

        /**
         * Execute the query with given query identifier.
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.QuerySample}
         * <p></p>
         * @param queryID id of the query
         * @return {@link com.trackplus.track.ws.beans.WSQueryResultItemBean}[] list of items 
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSQueryResultItemBean[] executeQuery(String queryID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Execute query with id: " + queryID);
            WSRequestString requestQueryID = new WSRequestString();
            requestQueryID.value = queryID;


            WSResponseQueryResult reply = trackWebService.executeQuery(requestQueryID);

            checkResponse(reply);
            return reply.reportBeans;
        }

        /**
         * Execute a custom query.
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.QuerySample}
         * <p></p>
         * @param  reportConfigBean the query definition bean for this query
         * @return an array of wrapped {@link com.trackplus.track.ws.beans.WSQueryResultItemBean} objects  
         * @throws TCLFacadeException 
         * @throws RemoteException 
         * 	
         */
        public WSQueryResultItemBean[] executeCustomQuery(WSQueryConfigBean reportConfigBean) /*throws RemoteException,TCLFacadeException*/{
            WSRequestQueryConfig requestReportConfig = new WSRequestQueryConfig();
            requestReportConfig.queryConfigBean = reportConfigBean;
            WSResponseQueryResult reply = trackWebService.executeCustomQuery(requestReportConfig);
            checkResponse(reply);
            return reply.reportBeans;
        }

        /**
         * <p>
         * Each query filter is made up of a number of elements, for example
         * which statuses shall be passed through the filter. All elements are
         * assembled in a report configuration bean, thus the report configuration
         * bean makes up the query filter.</p>
         * <p>
         * This method takes a report configuration bean as a parameter where
         * one can set the project(s). The report configuration bean returned will 
         * contain all possible options for this project pre-selection, 
         * similar to when the user selects a project or
         * release at the user interface.
         * </p>
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.QuerySample}
         * <p></p>
         * 
         * @param reportConfigBean here we set the project.
         * @param firstTime if firstTime is true select the firstProject
         *  
         * @return
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSQueryConfigBean loadQueryConfigBean(WSQueryConfigBean reportConfigBean, bool firstTime) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Load ReportConfigBean");

            WSRequestQueryConfig requestReportConfig = new WSRequestQueryConfig();
            requestReportConfig.queryConfigBean = reportConfigBean;
            requestReportConfig.firstTime = firstTime + "";


            WSResponseQueryConfig reply = trackWebService.loadQueryConfigBean(requestReportConfig);

            LOGGER.Debug("Load ReportConfigBean reply status=" + reply.status);
            checkResponse(reply);
            return reply.reportConfigBean;
        }

        //------------------- END OF Query handling ---------------------------------------------	


        //------------------- Item handling -----------------------------------------------------

        /**
         * This method makes the itemBean persistent (saves it to the database).
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.ItemSample}
         * <p></p>
         * @param itemBean
         * @return String  the id of the item
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public String saveItem(WSItemBean itemBean) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Saving item with synopsis: " + (itemBean == null ? "" : itemBean.synopsis));

            WSRequestItem requestItem = new WSRequestItem();
            requestItem.itemBean = itemBean;


            WSResponseString reply = trackWebService.saveItem(requestItem);

            LOGGER.Debug("Save item reply status=" + reply.status);
            checkResponse(reply);
            return reply.value;
        }


        /**
         * Create a copy of an issue. If <code>deepCopy</code> is true,
         * all history data is copied as well.
         * @param itemID
         * @param deepCopy
         * @return
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public String copyItem(String itemID, bool deepCopy) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Copy item:" + itemID);
            WSRequestCopyItem requestCopyItem = new WSRequestCopyItem();
            requestCopyItem.itemID = itemID;
            requestCopyItem.deepCopy = deepCopy + "";
            WSResponseString reply = trackWebService.copyItem(requestCopyItem);
            LOGGER.Debug("Copy item reply status=" + reply.status);
            checkResponse(reply);
            return reply.value;
        }

        public void moveItem(WSItemBean itemBean) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("saveMoveItem with synopsis: " + (itemBean == null ? "" : itemBean.synopsis));
            WSRequestItem requestItem = new WSRequestItem();
            requestItem.itemBean = itemBean;
            WSResponse reply = trackWebService.moveItem(requestItem);
            LOGGER.Debug("saveMoveItem reply status=" + reply.status);
            checkResponse(reply);
        }

        public void changeItemStatus(WSItemBean itemBean) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("saveChangeStatusItem with synopsis: " + itemBean.synopsis);
            WSRequestItem requestItem = new WSRequestItem();
            requestItem.itemBean = itemBean;
            WSResponse reply = trackWebService.changeItemStatus(requestItem);
            LOGGER.Debug("saveChangeStatusItem reply status=" + reply.status);
            checkResponse(reply);
        }

        /**
             * This method fetches a specific issue defined by its
             * object id from the database.
             * <h3>Example</h3> 
             * For an example see {@link com.trackplus.track.ws.ItemSample}
             * <p></p>
             * @param itemID
             *            the object id of the issue to be fetched from the database.
             * @return WSItemBean
             * @throws RemoteException 
             * @throws TCLFacadeException 
             */
        public WSItemBean getItem(String itemID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Getting item with id: " + itemID);

            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;

            WSResponseItem reply = trackWebService.getItem(requestItemID);

            LOGGER.Debug("Get item reply status=" + reply.status);
            checkResponse(reply);
            return reply.itemBean;
        }

        /**
         * This method fetches the history for a specific issue defined by its
         * object id from the database.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.ItemSample}
         * <p></p>
         * @param itemID
         *            the object id of the item to be fetched from the database.
         * @return WSItemHistoryBean
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSItemHistoryBean getItemHistory(String itemID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Getting item history with id: " + itemID);

            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;

            WSResponseItemHistory reply = trackWebService.getItemHistory(requestItemID);

            LOGGER.Debug("Get item history reply status=" + reply.status);
            checkResponse(reply);
            return reply.itemHistoryBean;
        }

        /**
         * This method retrieves the context of issues (related field and option values)
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.ItemSample}
         * <p></p>
         * @param itemIDs
         *            the object ids of the items to be fetched from the database.
         * @return WSItemBean
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSItemContextBean[] getItemContexts(String[] itemIDs) /*throws RemoteException, TCLFacadeException*/{
            if (itemIDs == null)
            {
                LOGGER.Debug("Getting items with null argument");
                return new WSItemContextBean[0];
            }
            LOGGER.Debug("Getting items with id: " + itemIDs.Length);

            WSRequestStringArray requestItemIDs = new WSRequestStringArray();
            requestItemIDs.values = itemIDs;

            WSResponseItemContextArray reply = trackWebService.getItemContexts(requestItemIDs);

            LOGGER.Debug("Get items reply status=" + reply.status);
            checkResponse(reply);
            return reply.itemContextBeans;
        }

        /**
         * Obtain an item and the display values for fields.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.ItemSample}
         * <p></p> 
         * @param itemID the id of the item
         * @return WSDisplayItemBean 
         * @throws RemoteException 
         */
        public WSDisplayItemBean getDisplayItem(String itemID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Getting showable item for id:" + itemID);

            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;

            WSResponseDisplayItem responseDisplayItem = trackWebService.getDisplayItem(requestItemID);

            LOGGER.Debug("Getting showable item response status=" + responseDisplayItem.status);
            checkResponse(responseDisplayItem);
            return responseDisplayItem.displayItemBean;
        }

        /**
         * Obtain the item context of a single issue.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.ItemSample}
         * <p></p> 
         * @param itemID the id of the item 
         * @return WSItemContextBean 
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSItemContextBean getItemContext(String itemID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the item context for item with id:" + itemID);

            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;

            WSResponseItemContext reply = trackWebService.getItemContext(requestItemID);

            LOGGER.Debug("Retrieving the  item context reply status=" + reply.status);
            checkResponse(reply);
            return reply.itemContextBean;
        }

        /**
         * Obtain the edit item context.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.ItemSample}
         * <p></p>
         * @param itemID the id of the item 
         * @return WSItemContextBean 
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSEditItemContextBean getEditItemContext(String itemID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the edit item context for item with id:" + itemID);

            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;

            WSResponseEditItemContext reply = trackWebService.getEditItemContext(requestItemID);

            LOGGER.Debug("Retrieving the edit item context reply status=" + reply.status);
            checkResponse(reply);
            return reply.editItemContextBean;
        }

        /**
         * Obtain the edit form (input mask definition).
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.FormSample}
         * <p></p>
         * @param projectID
         * @param issueTypeID
         * @return
         * @throws TCLFacadeException 
         * @throws RemoteException 
         */
        public WSFormBean getEditItemForm(String projectID, String issueTypeID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the edit form id:" + projectID + " and " + issueTypeID);

            WSRequestItemLocation requestItemLocation = new WSRequestItemLocation();
            requestItemLocation.projectID = projectID;
            requestItemLocation.issueTypeID = issueTypeID;

            WSResponseForm responseReturn = trackWebService.getEditItemForm(requestItemLocation);

            LOGGER.Debug("Retrieving the edit form responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.formBean;
        }

        /**
         * Obtain the context (option values) when creating an issue. The context
         * can depend on the project and issue type.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.ItemSample}
         * <p></p> 
         * @param projectID issueTypeID 
         * @return WSItemContextBean 
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSEditItemContextBean getNewItemContext(String projectID, String issueTypeID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Get the new item context for project: " + projectID + " and issueType: " + issueTypeID);

            WSRequestItemLocation requestItemLocation = new WSRequestItemLocation();
            requestItemLocation.projectID = projectID;
            requestItemLocation.issueTypeID = issueTypeID;


            WSResponseEditItemContext responseReturn = trackWebService.getNewItemContext(requestItemLocation);

            LOGGER.Debug("Get the new item context responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.editItemContextBean;
        }

        /**
         * Obtain the new issue form (input mask when creating an issue). The
         * form layout can depend on the project and issue type.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.FormSample}
         * <p></p>	 
         * @param projectID
         * @param issueTypeID
         * @return
         * @throws TCLFacadeException 
         * @throws RemoteException 
         */
        public WSFormBean getNewItemForm(String projectID, String issueTypeID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the new item form id:" + projectID + " and " + issueTypeID);

            WSRequestItemLocation requestItemLocation = new WSRequestItemLocation();
            requestItemLocation.projectID = projectID;
            requestItemLocation.issueTypeID = issueTypeID;


            WSResponseForm responseReturn = trackWebService.getNewItemForm(requestItemLocation);

            LOGGER.Debug("Retrieving the new item form responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.formBean;
        }

        /**
         * Obtain the context (option values) when creating a child issue. The context
         * can depend on the project and issue type.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.ItemSample}
         * <p></p> 
         * @param projectID issueTypeID 
         * @return WSItemContextBean 
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSEditItemContextBean getNewItemChildContext(String projectID, String issueTypeID, String parentID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Get the new item cjild context for project: " + projectID + " and issueType: " + issueTypeID);

            WSRequestNewItemChild requestNewItemChild = new WSRequestNewItemChild();
            requestNewItemChild.projectID = projectID;
            requestNewItemChild.issueTypeID = issueTypeID;
            requestNewItemChild.parentID = parentID;

            WSResponseEditItemContext responseReturn = trackWebService.getNewItemChildContext(requestNewItemChild);

            LOGGER.Debug("Get the new item context responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.editItemContextBean;
        }

        /**
         * Obtain the new child issue form (input mask when creating a child issue). The
         * form layout can depend on the project and issue type.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.FormSample}
         * <p></p>	
         * @param projectID
         * @param issueTypeID
         * @return
         * @throws TCLFacadeException 
         * @throws RemoteException 
         */
        public WSFormBean getNewItemChildForm(String projectID, String issueTypeID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the new item child form id:" + projectID + " and " + issueTypeID);

            WSRequestItemLocation requestItemLocation = new WSRequestItemLocation();
            requestItemLocation.projectID = projectID;
            requestItemLocation.issueTypeID = issueTypeID;

            WSResponseForm responseReturn = trackWebService.getNewItemChildForm(requestItemLocation);

            LOGGER.Debug("Retrieving the new item child form responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.formBean;
        }

        /**
         * Obtain the context when moving an issue to another project and issue type.
         * @param itemID
         * @param projectID
         * @param issueTypeID
         * @return
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSEditItemContextBean getMoveItemContext(String itemID, String projectID, String issueTypeID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Move item in project: " + projectID + " and issueType: " + issueTypeID);

            WSRequestMoveItem requestMoveItem = new WSRequestMoveItem();
            requestMoveItem.itemID = itemID;
            requestMoveItem.projectID = projectID;
            requestMoveItem.issueTypeID = issueTypeID;

            WSResponseEditItemContext responseReturn = trackWebService.getMoveItemContext(requestMoveItem);

            LOGGER.Debug("move item responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.editItemContextBean;
        }

        /**
         * Obtain the form (input mask) for moving an issue.
         * @param projectID
         * @param issueTypeID
         * @return
         * @throws TCLFacadeException 
         * @throws RemoteException 
         */
        public WSFormBean getMoveItemForm(String projectID, String issueTypeID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the move item form id:" + projectID + " and " + issueTypeID);

            WSRequestItemLocation requestItemLocation = new WSRequestItemLocation();
            requestItemLocation.projectID = projectID;
            requestItemLocation.issueTypeID = issueTypeID;

            WSResponseForm responseReturn = trackWebService.getMoveItemForm(requestItemLocation);

            LOGGER.Debug("Retrieving the move item form responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.formBean;
        }

        /**
         * Obtain the change status item context. 
         * @param itemID the id of the item 
         * @return WSItemContextBean 
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSEditItemContextBean getChangeStatusItemContext(String itemID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the change status item context for item with id:" + itemID);

            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;

            WSResponseEditItemContext reply = trackWebService.getChangeStatusItemContext(requestItemID);

            LOGGER.Debug("Retrieving the change status item context reply status=" + reply.status);
            checkResponse(reply);
            return reply.editItemContextBean;
        }

        /**
         * Obtain the ChangeStatus item form.
         * @param projectID
         * @param issueTypeID
         * @return
         * @throws TCLFacadeException 
         * @throws RemoteException 
         */
        public WSFormBean getChangeStatusItemForm(String projectID, String issueTypeID) /*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("Retrieving the ChangeStatus item form id:" + projectID + " and " + issueTypeID);

            WSRequestItemLocation requestItemLocation = new WSRequestItemLocation();
            requestItemLocation.projectID = projectID;
            requestItemLocation.issueTypeID = issueTypeID;

            WSResponseForm responseReturn = trackWebService.getChangeStatusItemForm(requestItemLocation);

            LOGGER.Debug("Retrieving the ChangeStatus item form responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.formBean;
        }



        /**
         * Get used projects this user has access to.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.samples.BaseOptionSample}.<br>
         * Another example can be found in {@link com.trackplus.track.ws.samples.ItemSample}
         * <p></p>	
         * @return list of label value beans with projects
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSLabelValueBean[] getProjects() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get the projects");
            WSResponseLabelValueBeanArray responseReturn = trackWebService.getProjects();
            LOGGER.Debug("Get the projects responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.labelValueBeans;
        }

        /**
         * Get used projects this user has access to.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.samples.BaseOptionSample}.<br>
         * Another example can be found in {@link com.trackplus.track.ws.samples.ItemSample}
         * <p></p>	
         * @return list of label value beans with projects
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSLabelValueBean[] getAllProjects() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get all projects");
            WSResponseLabelValueBeanArray responseReturn = trackWebService.getAllProjects();
            LOGGER.Debug("Get the projects responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.labelValueBeans;
        }

        /**
         * Get all projects as tree this user has access to.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.samples.BaseOptionSample}.<br>
         * Another example can be found in {@link com.trackplus.track.ws.samples.ItemSample}
         * <p></p>	
         * @return list of label value beans with projects
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSTreeNode[] getProjectsAsTree() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get the projects as tree");
            WSResponseTreeNodeArray responseReturn = trackWebService.getProjectsAsTree();
            LOGGER.Debug("Get the projects as tree responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.nodes;
        }


        public WSProjectEditWithContext getNewProjectWithContext() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting new edit project");
            WSResponseProjectEditWithContext reply = trackWebService.getNewProjectWithContext();
            LOGGER.Debug("Get new edit project reply status=" + reply.status);
            checkResponse(reply);
            return reply.projectEditWithContext;
        }

        public WSProjectEditBean getEditProject(String projectID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting edit project with id: " + projectID);
            WSRequestString requestProjectID = new WSRequestString();
            requestProjectID.value = projectID;
            WSResponseProjectEdit reply = trackWebService.getEditProject(requestProjectID);
            LOGGER.Debug("Get edit project reply status=" + reply.status);
            checkResponse(reply);
            return reply.projectEdit;
        }
        public WSProjectEditWithContext getEditProjectWithContext(String projectID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting edit project context with id: " + projectID);
            WSRequestString requestProjectID = new WSRequestString();
            requestProjectID.value = projectID;
            WSResponseProjectEditWithContext reply = trackWebService.getEditProjectWithContext(requestProjectID);
            LOGGER.Debug("Get edit project reply status=" + reply.status);
            checkResponse(reply);
            return reply.projectEditWithContext;
        }

        public String saveProject(WSProjectEditBean projectEditBean)/*throws RemoteException,TCLFacadeException*/{
            WSRequestProjectEdit requestProjectEdit = new WSRequestProjectEdit();
            requestProjectEdit.projectEdit = projectEditBean;
            WSResponseString reply = trackWebService.saveProject(requestProjectEdit);
            LOGGER.Debug("Save Project reply status=" + reply.status);
            checkResponse(reply);
            return reply.value;
        }
        public void deleteProject(String projectID)/*throws RemoteException,TCLFacadeException*/{
            WSRequestString requestString = new WSRequestString();
            requestString.value = projectID;
            WSResponse responseReturn = trackWebService.deleteProject(requestString);
            LOGGER.Debug("Delete the project responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }


        /**
         * Get all projects as tree this user has access to.
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.samples.BaseOptionSample}.<br>
         * Another example can be found in {@link com.trackplus.track.ws.samples.ItemSample}
         * <p></p>	
         * @return list of label value beans with projects
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSTreeNode[] getProjectsWithCreateItemRightForItemType(String issueTypeID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("getdProjectsWithCreateIssueRightForItemType for project: " + issueTypeID);
            WSRequestString requestIssueType = new WSRequestString();
            requestIssueType.value = issueTypeID;
            WSResponseTreeNodeArray responseReturn = trackWebService.getProjectsWithCreateItemRightForItemType(requestIssueType);
            LOGGER.Debug("getdProjectsWithCreateIssueRightForItemType responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.nodes;
        }

        /**
         * Get all issue types for a given project. 
         * <h3>Example</h3> 
         * For an example see {@link com.trackplus.track.ws.samples.BaseOptionSample}.<br>
         * Another example can be found in {@link com.trackplus.track.ws.samples.ItemSample}
         * <p></p>	
         * @param projectID  the project id
         * @return list of label value beans with issue types
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSLabelValueBean[] getIssueTypes(String projectID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting the issue types for project: " + projectID);
            WSRequestString requestProject = new WSRequestString();
            requestProject.value = projectID;
            WSResponseLabelValueBeanArray responseReturn = trackWebService.getIssueTypes(requestProject);
            LOGGER.Debug("Getting the issue types responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.labelValueBeans;
        }

        /**
         * Get the sub-projects from given project.
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.BaseOptionSample}.
         * <p></p>	
         * @param projectID  the project id
         * @return list of label value beans with subsystems.
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public WSLabelValueBean[] getSubprojects(String projectID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting the subprojects for project: " + projectID);
            WSRequestString requestProject = new WSRequestString();
            requestProject.value = projectID;
            WSResponseLabelValueBeanArray responseReturn = null; // TODO trackWebService.getSubprojects(requestProject);
            LOGGER.Debug("Getting the subprojects responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.labelValueBeans;
        }

        /**
         * Gets the BaseDropDown container: 
         *   contains all projects, issue types, priorities, statuses.
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.BaseOptionSample}.
         * <p></p>	
         * @return a complete container with all options.
         */
        public WSBaseOptionsContainer getBaseOptionsContainer()/*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("getBaseDropDownContainer ");
            WSResponseBaseOptionsContainer responseReturn = trackWebService.getBaseOptionsContainer();
            LOGGER.Debug("getBaseDropDownContainer responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.baseOptionsContainer;
        }

        /**
         * Obtain all custom fields defined.
         * <h4>Example</h4> 
         * For an example see {@link com.trackplus.track.ws.samples.ItemSample}.
         * <p></p>	 
         * @return an array with custom fields.
         */
        public WSField[] getCustomFields()/*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("getCustomFields ");
            WSResponseFieldArray responseReturn = trackWebService.getCustomFields();
            LOGGER.Debug("getCustomFields responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.fields;
        }

        //------------------- Attachment handling ----------------------------

        //	/**
        //	 * Get the attachments for an item.
        //	 * @param itemID  the item id
        //	 * @return WSAttachmentBean[] 
        //	 * @throws TCLFacadeException 
        //	 * @throws RemoteException 
        //	 */
        //	public WSAttachmentBean[] getAttachments(String itemID) /*throws RemoteException,TCLFacadeException*/{
        //		LOGGER.Debug("Getting the attachments for item with id: "+itemID);
        //		
        //		WSRequestString requestItemID=new WSRequestString();
        //		requestItemID.setValue(itemID);
        //		
        //		GetAttachments getAttachments=new GetAttachments();
        //		getAttachments.setRequestItemID(requestItemID);
        //		
        //		GetAttachmentsResponse response=trackWebService.getAttachments(getAttachments);
        //		WSResponseAttachmentArray responseReturn=response.get_return();
        //		
        //		LOGGER.Debug("Getting the attachments responseReturn status="+responseReturn.status);
        //		checkResponse(responseReturn);
        //		return responseReturn.getAttachmentBeans();
        //	}

        /**
            * Save an attachment.
            * @param attach
            * @return
            * @throws TCLFacadeException 
            */
        public WSAttachmentBean saveAttachment(WSAttachmentBean attach)/*throws RemoteException, TCLFacadeException*/{
            String filename = attach.fileName;
            FileStream fs = new FileStream(filename, FileMode.OpenOrCreate, FileAccess.Read);
            byte[] binaryData = new byte[fs.Length];
            fs.Read(binaryData, 0, System.Convert.ToInt32(fs.Length));
            fs.Close();

            return saveAttachment(attach, binaryData);
        }

        /**
         * Save an attachment from binaryData.
         * @param attach
         * @param binaryData
         * @return
         * @throws RemoteException
         * @throws TCLFacadeException 
         */
        public WSAttachmentBean saveAttachment(WSAttachmentBean attach, byte[] binaryData)/*throws RemoteException, TCLFacadeException*/{
            LOGGER.Debug("TCLFacde saveAttachment(WSAttachmentBean attach,InputStream is)");

            WSRequestAttachment requestAttachment = new WSRequestAttachment();
            requestAttachment.attachmentBean = attach;

            WSResponseAttachment responseReturn = trackWebService.saveAttachment(requestAttachment, binaryData);
            LOGGER.Debug("Save the attachment responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.attachmentBean;
        }

        /**
         * Delete an attachment.
         * @param attachmentBean
         * @throws RemoteException 
         * @throws TCLFacadeException 
         */
        public void deleteAttachment(WSAttachmentBean attachmentBean) /*throws RemoteException,TCLFacadeException*/{
            ///	LOGGER.Debug("Delete the attachment for item id: "+attachmentBean.getItemKey()+" with name="+attachmentBean.getFileName());
            WSRequestAttachment requestAttachment = new WSRequestAttachment();
            requestAttachment.attachmentBean = attachmentBean;
            WSResponse responseReturn = trackWebService.deleteAttachment(requestAttachment);
            LOGGER.Debug("Delete the attachment responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);

        }

        /**
         * Download the attachment.
         * @param attach
         * @return
         * @throws RemoteException
         * @throws TCLFacadeException 
         */
        public byte[] downloadAttachment(WSAttachmentBean attach) /*throws RemoteException, TCLFacadeException*/ {
            WSRequestAttachment requestAttachment = new WSRequestAttachment();
            requestAttachment.attachmentBean = attach;


            WSResponseDownloadAttachment responseReturn = trackWebService.downloadAttachment(requestAttachment);
            checkResponse(responseReturn);

            return responseReturn.binaryData;
        }

        /**
         * Obtain a list of comments for a specific issue.
         * @param itemID the issue object identifier for which the comments shall be retrieved.
         * @return an array of comments for this issue.
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public WSCommentEdit[] getComments(String itemID)/*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get the comments for item id: " + itemID);
            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;
            WSResponseCommentArray reply = trackWebService.getComments(requestItemID);
            LOGGER.Debug("Get item commentsreply status=" + reply.status);
            checkResponse(reply);
            return reply.comments;
        }

        /**
         * Delete comments for an issue. 
         * @param itemID the issue object identifier for which comments shall be deleted.
         * @param commentIDs an array with comment object identifiers for comments that shall be deleted.
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public void deleteComments(String itemID, String[] commentIDs) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Delete the comments for item id: " + itemID);
            if (commentIDs == null || commentIDs.Length == 0)
            {
                LOGGER.Debug("Emty delete comments");
                return;
            }
            WSRequestDeleteComments requestDeleteComments = new WSRequestDeleteComments();
            requestDeleteComments.workItemID = itemID;
            requestDeleteComments.commentIDs = commentIDs;
            WSResponse responseReturn = trackWebService.deleteComments(requestDeleteComments);
            LOGGER.Debug("Delete the comments responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }

        /**
         * Create/modify a comment
         * @param comment -The comment contains the issue object identifier this comment 
         * belongs to and the comment text.
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public void editComment(WSCommentEdit comment) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Edit the comment");
            WSRequestComment requestComment = new WSRequestComment();
            requestComment.comment = comment;
            WSResponse responseReturn = trackWebService.editComment(requestComment);
            LOGGER.Debug("Edit the comment responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }

        /**
     * Obtain a list of links for a specific item.
     * @param itemID the item object identifier for which the links shall be retrieved.
     * @return an array of links for this item.
     * @throws RemoteException
     * @throws TCLFacadeException
     */
        public WSItemLinkBean[] getItemLinks(String itemID) /* throws RemoteException, TCLFacadeException */{
            LOGGER.Debug("Get the item links for item id: " + itemID);

            WSRequestString requestItemID = new WSRequestString();
            requestItemID.value = itemID;
            WSResponseItemLinkArray response = trackWebService.getItemLinks(requestItemID);

            LOGGER.Debug("Get item links reply status=" + response.status);
            checkResponse(response);
            return response.itemLinkBeans;
        }

        /**
         * Get a link for the given id
         * @param itemLinkID
         * @return
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public WSItemLinkBean getItemLink(String workItemID, String itemLinkID) /*throws RemoteException, TCLFacadeException */ {
            LOGGER.Debug("Get the item link for item id: " + itemLinkID);

            WSRequestItemLink requestItemLink = new WSRequestItemLink();
            requestItemLink.workItemID = workItemID;
            requestItemLink.itemLinkID = itemLinkID;

            WSResponseItemLink response = trackWebService.getItemLink(requestItemLink);

            LOGGER.Debug("Get item link response status=" + response.status);
            checkResponse(response);
            return response.itemLinkBean;
        }

        /**
         * Delete links for an item.
         * @param itemID the item object identifier for which comments shall be deleted.
         * @param itemLinkIDs an array with link object identifiers for links that shall be deleted.
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public void deleteItemLinks(String itemID, String[] itemLinkIDs) /* throws RemoteException, TCLFacadeException */{
            LOGGER.Debug("Delete the links for item id: " + itemID);
            if (itemLinkIDs == null || itemLinkIDs.Length == 0)
            {
                LOGGER.Debug("Emty delete links");
                return;
            }
            WSRequestDeleteItemLinks requestDeleteItemLinks = new WSRequestDeleteItemLinks();
            requestDeleteItemLinks.workItemID = itemID;
            requestDeleteItemLinks.itemLinkIDs = itemLinkIDs;

            WSResponse response = trackWebService.deleteItemLinks(requestDeleteItemLinks);
            LOGGER.Debug("Delete the item links response status=" + response.status);
            checkResponse(response);
        }
        /**
         * Create/modify a item link
         * @param workItemID the id of the work item
         * @param itemLinkBean -The item link to save
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public String saveItemLink(String workItemID, WSItemLinkBean itemLinkBean) /* throws RemoteException,
			TCLFacadeException */{
                                     LOGGER.Debug("Saving item link for item id:" + workItemID);

                                     WSRequestItemLinkEdit requestItemLinkEdit = new WSRequestItemLinkEdit();
                                     requestItemLinkEdit.workItemID = workItemID;
                                     requestItemLinkEdit.itemLinkBean = itemLinkBean;

                                     WSResponseString response = trackWebService.saveItemLink(requestItemLinkEdit);

                                     LOGGER.Debug("Save item link response status=" + response.status);
                                     checkResponse(response);
                                     return response.value;
                                 }

        /**
         * Get all link types.
         * @return WSOptionBean[]
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public WSOptionBean[] getLinkTypes() /* throws RemoteException, TCLFacadeException */{
            LOGGER.Debug("Get all link types");

            WSResponseOptionArray response = trackWebService.getLinkTypes();

            LOGGER.Debug("Get all link types response status=" + response.status);
            checkResponse(response);
            return response.options;
        }

        /**
         * Get parameters definition for link type
         * @param linkTypeID
         * @return
         */
        public WSParameter[] getLinkTpeParameters(String linkTypeID) /* throws RemoteException, TCLFacadeException */{
            LOGGER.Debug("Get linkType parameters");

            WSRequestString requestLinkTypeID = new WSRequestString();
            requestLinkTypeID.value = linkTypeID;

            WSResponseParameterArray response = trackWebService.getLinkTpeParameters(requestLinkTypeID);

            LOGGER.Debug("Get link type parameters response status=" + response.status);
            checkResponse(response);
            return response.parameters;
        }



        /**
         * Get all persons.
         * @return
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public WSPersonBean[] getAllPersons() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get all persons");
            WSResponsePersonArray responseReturn = trackWebService.getAllPersons();
            LOGGER.Debug("Get all persons  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.persons;
        }
        /**
         * Get persons in the group
         * @param groupID
         * @return
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public WSPersonBean[] getPersonsForGroup(String groupID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get persons in group");
            WSRequestString requestGroupID = new WSRequestString();
            requestGroupID.value = groupID;
            WSResponsePersonArray responseReturn = trackWebService.getPersonsForGroup(requestGroupID);
            LOGGER.Debug("Get persons for group  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.persons;
        }

        public WSPersonEditWithContext getNewPersonWithContext() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting new edit person");
            WSResponsePersonEditWithContext reply = trackWebService.getNewPersonWithContext();
            LOGGER.Debug("Get new  person context reply status=" + reply.status);
            checkResponse(reply);
            return reply.personEditWithContext;
        }


        /**
         * @param personID
         * @return a person bean for editing
         */
        public WSPersonEditBean getEditPerson(String personID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting edit person with id: " + personID);
            WSRequestString requestPersonID = new WSRequestString();
            requestPersonID.value = personID;
            WSResponsePersonEdit reply = trackWebService.getEditPerson(requestPersonID);
            LOGGER.Debug("Get edit person reply status=" + reply.status);
            checkResponse(reply);
            return reply.personEdit;
        }

        /**
     * 
     * @param loginName
     * @return WSPersonEditBean
     * @throws RemoteException
     * @throws TCLFacadeException
     */
        public WSPersonEditBean getEditPersonByLoginName(String loginName) /* throws RemoteException, TCLFacadeException */{
            LOGGER.Debug("Getting edit person with name: " + loginName);

            WSRequestString requestPerson = new WSRequestString();
            requestPerson.value = loginName;

            WSResponsePersonEdit response = trackWebService.getEditPersonByLoginName(requestPerson);

            LOGGER.Debug("Get edit person by login name response status=" + response.status);
            checkResponse(response);
            return response.personEdit;
        }


        /**
         * @param personID
         * @return a person context bean for editing
         */
        public WSPersonEditWithContext getEditPersonWithContext(String personID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting edit person comtext with id: " + personID);
            WSRequestString requestPersonID = new WSRequestString();
            requestPersonID.value = personID;
            WSResponsePersonEditWithContext reply = trackWebService.getEditPersonWithContext(requestPersonID);
            LOGGER.Debug("Get edit person context reply status=" + reply.status);
            checkResponse(reply);
            return reply.personEditWithContext;
        }

        /**
         * save a person
         * @param personEdit
         * @return the object id of the person saved or an error message
         */
        public String savePerson(WSPersonEditBean personEdit) /*throws RemoteException,TCLFacadeException*/{
            WSRequestPersonEdit requestPersonEdit = new WSRequestPersonEdit();
            requestPersonEdit.personEdit = personEdit;
            WSResponseString reply = trackWebService.savePerson(requestPersonEdit);
            LOGGER.Debug("Save person reply status=" + reply.status);
            checkResponse(reply);
            return reply.value;
        }

        /**
         * delete a person by id
         * @param personID
         */
        public void deletePerson(String personID)  /*throws RemoteException,TCLFacadeException*/{
            WSRequestString requestString = new WSRequestString();
            requestString.value = personID;
            WSResponse responseReturn = trackWebService.deletePerson(requestString);
            LOGGER.Debug("Delete the person responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }
        /**
         * delete a person and replace all dependencies with other person
         * @param personID
         * @param newPersonID
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public void replaceAndDeletePerson(String personID, String newPersonID)  /*throws RemoteException,TCLFacadeException*/{
            WSRequestReplaceAndDelete request = new WSRequestReplaceAndDelete();
            request.objectID = personID;
            request.newObjectID = newPersonID;
            WSResponse responseReturn = trackWebService.replaceAndDeletePerson(request);
            LOGGER.Debug("Replcae and delete the person responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }




        public WSGroupBean[] getAllGroups() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get all groups");
            WSResponseGroupArray responseReturn = trackWebService.getAllGroups();
            LOGGER.Debug("Get all groups responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.groups;
        }


        /**
         * @param groupID
         * @return a group bean for editing
         */
        public WSGroupEditBean getEditGroup(String groupID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Getting edit group with id: " + groupID);
            WSRequestString requestGroupID = new WSRequestString();
            requestGroupID.value = groupID;
            WSResponseGroupEdit reply = trackWebService.getEditGroup(requestGroupID);
            LOGGER.Debug("Get edit group reply status=" + reply.status);
            checkResponse(reply);
            return reply.groupEdit;
        }

        /**
         * save a group
         * @param groupEdit
         * @return the object id of the group saved or an error message
         */
        public String saveGroup(WSGroupEditBean groupEdit) /*throws RemoteException,TCLFacadeException*/{
            WSRequestGroupEdit requestGroupEdit = new WSRequestGroupEdit();
            requestGroupEdit.groupEdit = groupEdit;
            WSResponseString reply = trackWebService.saveGroup(requestGroupEdit);
            LOGGER.Debug("Save group reply status=" + reply.status);
            checkResponse(reply);
            return reply.value;
        }

        /**
         * delete a group by id
         * @param groupID
         */
        public void deleteGroup(String groupID)  /*throws RemoteException,TCLFacadeException*/{
            WSRequestString requestString = new WSRequestString();
            requestString.value = groupID;
            WSResponse responseReturn = trackWebService.deleteGroup(requestString);
            LOGGER.Debug("Delete the group responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }
        /**
         * Delete a group and replace with othe one
         * @param groupID
         * @param newGroupID
         * @throws RemoteException
         * @throws TCLFacadeException
         */
        public void replaceAndDeleteGroup(String groupID, String newGroupID)  /*throws RemoteException,TCLFacadeException*/{
            WSRequestReplaceAndDelete request = new WSRequestReplaceAndDelete();
            request.objectID = groupID;
            request.newObjectID = newGroupID;
            WSResponse responseReturn = trackWebService.replaceAndDeleteGroup(request);
            LOGGER.Debug("Replace and delete the Group responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }
        /**
         * Add person(s) to group
         * @param groupID the id of group
         * @param personID the ids of persons
         */
        public void addPersonsToGroup(String groupID, String[] personIDs)/*throws RemoteException,TCLFacadeException*/{
            WSRequestGroupAssigment requestGroupAssigment = new WSRequestGroupAssigment();
            requestGroupAssigment.groupID = groupID;
            requestGroupAssigment.personIDs = personIDs;
            WSResponse responseReturn = trackWebService.addPersonsToGroup(requestGroupAssigment);
            LOGGER.Debug("Add person to group responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }

        /**
         * Remove person(s) from group
         * @param groupID the id of group
         * @param personID the ids of persons
         */
        public void removePersonsFromGroup(String groupID, String[] personIDs)/*throws RemoteException,TCLFacadeException*/{
            WSRequestGroupAssigment requestGroupAssigment = new WSRequestGroupAssigment();
            requestGroupAssigment.groupID = groupID;
            requestGroupAssigment.personIDs = personIDs;
            WSResponse responseReturn = trackWebService.removePersonsFromGroup(requestGroupAssigment);
            LOGGER.Debug("Remove person from group responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
        }

        /**
         * Get all roles
         * @return
         */
        public WSLabelValueBean[] getAllRoles() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get all roles");
            WSResponseLabelValueBeanArray responseReturn = trackWebService.getAllRoles();
            LOGGER.Debug("Get all roles  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.labelValueBeans;
        }
        /**
         * Get all assigned roles for project
         * @param requestProjectID
         * @return
         */
        public WSRoleAssignmentBean[] getAssignmentRolesByProject(String projectID)/*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get project lists");
            WSRequestString requestString = new WSRequestString();
            requestString.value = projectID;
            WSResponseRoleAssignmentArray responseReturn = trackWebService.getAssignmentRolesByProject(requestString);
            LOGGER.Debug("Get project lists  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.roleAssignments;
        }

        /**
         * add a role assignment
         * @param requestRoleAssignmentEdit
         * @return
         */
        public void addRoleAssignment(WSRoleAssignmentEditBean roleAssignmentEditBean)/*throws RemoteException,TCLFacadeException*/{
            WSRequestRoleAssignmentEdit requestRoleAssignmentEdit = new WSRequestRoleAssignmentEdit();
            requestRoleAssignmentEdit.roleAssignmentEdit = roleAssignmentEditBean;
            WSResponse reply = trackWebService.addRoleAssignment(requestRoleAssignmentEdit);
            LOGGER.Debug("add role assignment reply status=" + reply.status);
            checkResponse(reply);
        }

        /**
         * remove a role assignment
         * @param requestRoleAssignmentEdit
         * @return
         */
        public void removeRoleAssignment(WSRoleAssignmentEditBean roleAssignmentEditBean)/*throws RemoteException,TCLFacadeException*/{
            WSRequestRoleAssignmentEdit requestRoleAssignmentEdit = new WSRequestRoleAssignmentEdit();
            requestRoleAssignmentEdit.roleAssignmentEdit = roleAssignmentEditBean;
            WSResponse reply = trackWebService.removeRoleAssignment(requestRoleAssignmentEdit);
            LOGGER.Debug("remove role assignment reply status=" + reply.status);
            checkResponse(reply);
        }




        public WSListBean[] getGlobalLists() /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get global lists");
            WSResponseListArray responseReturn = trackWebService.getGlobalLists();
            LOGGER.Debug("Get global lists  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.lists;
        }
        public WSListBean[] getProjectLists(String projectID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get project lists");
            WSRequestString requestString = new WSRequestString();
            requestString.value = projectID;
            WSResponseListArray responseReturn = trackWebService.getProjectLists(requestString);
            LOGGER.Debug("Get project lists  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.lists;
        }

        public WSListBean loadList(String listID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("load list");
            WSRequestString requestString = new WSRequestString();
            requestString.value = listID;
            WSResponseList responseReturn = trackWebService.loadList(requestString);
            LOGGER.Debug("load list  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.list;
        }

        public WSListOptionBean[] getListOptions(String listID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get list options ");
            WSRequestString requestString = new WSRequestString();
            requestString.value = listID;
            WSResponseListOptionArray responseReturn = trackWebService.getListOptions(requestString);
            LOGGER.Debug("Get list options  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.options;
        }

        public WSListOptionBean[] getListOptionsByParent(String parentID) /*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("Get list options by paret ");
            WSRequestString requestString = new WSRequestString();
            requestString.value = parentID;
            WSResponseListOptionArray responseReturn = trackWebService.getListOptionsByParent(requestString);
            LOGGER.Debug("Get list options by parent  responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.options;
        }

        public String saveList(WSListBean listBean)/*throws RemoteException,TCLFacadeException*/{
            WSRequestList requestList = new WSRequestList();
            requestList.list = listBean;
            WSResponseString reply = trackWebService.saveList(requestList);
            LOGGER.Debug("Save list reply status=" + reply.status);
            checkResponse(reply);
            return reply.value;
        }

        public void deleteList(String listID)/*throws RemoteException,TCLFacadeException*/{
            WSRequestString requestString = new WSRequestString();
            requestString.value = listID;
            WSResponse responseReturn = trackWebService.deleteList(requestString);
            LOGGER.Debug("Delete the list responseReturn status=" + responseReturn.status);
            checkResponse(responseReturn);

        }

        public WSListOptionBean[] clearListOptions(String listID)/*throws RemoteException,TCLFacadeException*/{
            LOGGER.Debug("clear list options ");
            WSRequestString requestString = new WSRequestString();
            requestString.value = listID;
            WSResponseListOptionArray responseReturn = trackWebService.clearListOptions(requestString);
            //		LOGGER.Debug("clear list options  responseReturn status="+responseReturn.status);
            checkResponse(responseReturn);
            return responseReturn.options;
        }

        public void addListOptions(WSListOptionBean[] options)/*throws RemoteException,TCLFacadeException*/{
            WSRequestOptionArray requestOptions = new WSRequestOptionArray();
            requestOptions.options = options;
            WSResponse reply = trackWebService.addListOptions(requestOptions);
            //		LOGGER.Debug("add list options reply status="+reply.status);
            checkResponse(reply);
        }

        public String saveListOption(WSListOptionBean option) /*throws RemoteException,TCLFacadeException*/{
            WSRequestOption requestOption = new WSRequestOption();
            requestOption.option = option;
            WSResponseString reply = trackWebService.saveListOption(requestOption);
            //		LOGGER.Debug("Save option list reply status="+reply.status);
            checkResponse(reply);
            return reply.value;
        }

        public void deletListOption(String optionID) /*throws RemoteException,TCLFacadeException*/{
            WSRequestString requestString = new WSRequestString();
            requestString.value = optionID;
            WSResponse responseReturn = trackWebService.deleteListOption(requestString);
            //		LOGGER.Debug("Delete the list option responseReturn status="+responseReturn.status);
            checkResponse(responseReturn);
        }

        //---------------------------private methods
        private void checkResponse(WSResponse responseReturn) /*throws TCLFacadeException*/ {
            LOGGER.Debug("Checking responseReturn...\n");
            if (!STATUS_OK.Equals(responseReturn.status))
            {
                //there is a error on server side.

                TCLFacadeException serverException = new TCLFacadeException(responseReturn.errorMessages[0]);
                serverException.setStatusCode(responseReturn.status);
                int errorCode = 0;
                try
                {

                    if (responseReturn.errorCode != null && !"".Equals(responseReturn.errorCode))

                    {

                        errorCode = int.Parse(responseReturn.errorCode);

                    }
                }
                catch (Exception ex)
                {
                    LOGGER.Debug(ex.Message.ToString());
                }
                for (int i=0; i < responseReturn.errorMessages.Length; ++i) {
                    LOGGER.Error( responseReturn.errorMessages[i] );
                }
                serverException.setErrorCode(errorCode);
                serverException.setErrorMessages(responseReturn.errorMessages);
                throw serverException;
            }
        }
    }
}
