/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2014 Steinbeis GmbH & Co. KG
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id: BaseOptionSample.cs 1465 2015-10-01 11:56:46Z friedric $
 */
using System;
using System.Collections.Generic;
using System.Text;
using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;

namespace com.trackplus.track.ws
{
    /**
     * <p>
     * In some input forms you may define fields where the user can select
     * from a number of options, like status, priority, manager or responsible. </p>
     * <p>
     * This class demonstrates how to retrieve all available option fields
     * and their possible values via the web service interface.
     * </p>
     * <p>
     * The class uses the {@link LoginSample} class to perform
     * the login. You need to set the URL and the user name and password in that
     * class before you compile and run this class here.</p>
     * <p>
     * This example is a C# console application. You can run it directly by
     * calling its <code>runBaseOptionSample()</code> method.</p>
     */
    public class BaseOptionSample : LoginSample
    {
        public BaseOptionSample()
            : base()
        {

        }
        public BaseOptionSample(TCLFacade connector)
            : base(connector)
        {
        }
        public static void printNode(WSTreeNode node,String indent){
            Console.WriteLine(indent + "[id:" + node.id + ", label=" + node.label + ", icon=" + node.icon + "]");
		    WSTreeNode[] childern=node.children;
		    if(childern!=null&&childern.Length>0){
			    for(int i=0;i<childern.Length;i++){
				    printNode(childern[i],indent+"\t");
			    }
		    }
	    }

        /**
         * Obtains the projects from the server
         * 
         */
        public void getProjectsIssueTypes()
        {
            Console.WriteLine("\n->BaseOptionSample.getProjectsIssueTypes");
            try
            {
                WSLabelValueBean[] projects = connector.getProjects();
                if (projects == null || projects.Length == 0)
                {
                    Console.WriteLine("No projects found on the server. Maybe the user does not have any rights in any project");
                }
                else
                {
                    Console.WriteLine("Successfully obtained " + projects.Length + " projects:");
                    for (int i = 0; i < projects.Length; i++)
                    {
                        String projectID = projects[i].value;
                        Console.WriteLine("Project: ID=" + projectID + " Label=" + projects[i].label);
                        WSLabelValueBean[] issueTypes = connector.getIssueTypes(projectID);
                        for (int j = 0; j < issueTypes.Length; j++)
                        {
                            Console.WriteLine("\t IssueType: ID=" + issueTypes[j].value + " Label=" + issueTypes[j].label);
                        }
                    }
                    Console.WriteLine();
                }
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-BaseOptionSample.getProjectsIssueTypes()\n");
        }
        /**
         * Getting the base options:
         *	all projects;
         *	all issueTypes;
         *	all priorities;
         *	all severities;
         *	all states;
         */
        public void getBaseOptionsContainer()
        {
            Console.WriteLine("\n->BaseOptionSample.getBaseOptionsContainer()");
            try
            {
                WSBaseOptionsContainer baseOptionsContainer = connector.getBaseOptionsContainer();
                //all projects
                WSOptionBean[] allProjects = baseOptionsContainer.projects;
                printOptions("Project", allProjects);
                Console.WriteLine();

                //all issue types
                WSSortedOptionBean[] allIssueTypes = baseOptionsContainer.issueTypes;
                printSortedOptions("IssueType", allIssueTypes);
                Console.WriteLine();

                //all priorities
                WSSortedOptionBean[] allPriorities = baseOptionsContainer.priorities;
                printSortedOptions("Priority", allPriorities);
                Console.WriteLine();

                //all severities
                WSSortedOptionBean[] allSeverities = baseOptionsContainer.severities;
                printSortedOptions("Severity", allSeverities);
                Console.WriteLine();

                //all states
                WSStateOptionBean[] allStates = baseOptionsContainer.states;
                printStateOptions("State", allStates);
                Console.WriteLine();
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-BaseOptionSample.getBaseOptionsContainer()\n");
        }
        private void printOptions(String optionType, WSOptionBean[] options)
        {
            for (int i = 0; i < options.Length; i++)
            {
                Console.WriteLine(optionType + ": ID=" + options[i].objectID + " Label=" + options[i].label);
            }
        }
        private void printSortedOptions(String optionType, WSSortedOptionBean[] options)
        {
            for (int i = 0; i < options.Length; i++)
            {
                Console.WriteLine(optionType + ": ID=" + options[i].objectID + " Label=" + options[i].label + " SortOrder=" + options[i].sortOrderValue);
            }
        }
        private void printStateOptions(String optionType, WSStateOptionBean[] options)
        {
            for (int i = 0; i < options.Length; i++)
            {
                Console.WriteLine(optionType + ": ID=" + options[i].objectID + " Label=" + options[i].label + " SortOrder=" + options[i].sortOrderValue + " StrateFlag=" + options[i].stateFlag);
            }
        }

        public void getProjectsAsTree() 
        {
		    Console.WriteLine("\n->BaseOptionSample.getProjectsAsTree()");
		    try {
			    WSTreeNode[] projects = connector.getProjectsAsTree();
			    if (projects == null || projects.Length == 0) {
				    Console.WriteLine("No projects found on the server. Maybe the user does not have right in any project");
			    } else {
                    Console.WriteLine("Successfully obtained " + projects.Length + " projects:");
                    for (int i = 0; i < projects.Length; i++)
                    {
					    printNode(projects[i], "");
				    }
				    Console.WriteLine();
			    }
		    } catch (TCLFacadeException e) {
			    handleTCLFacadeException(e);
		    }
            Console.WriteLine("<-BaseOptionSample.getProjectsAsTree()\n");
        }


        public static void runBaseOptionSample(String[] args)
        {
            BaseOptionSample baseOptionSample = new BaseOptionSample();
            baseOptionSample.login();
            
            //get projects as Tree
            baseOptionSample.getProjectsAsTree();

            //get projects and issueTypes
            baseOptionSample.getProjectsIssueTypes();

            //base drop-down container
            baseOptionSample.getBaseOptionsContainer();
        }
    }
}
