/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2014 Steinbeis GmbH & Co. KG
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id: ItemSample.cs 1465 2015-10-01 11:56:46Z friedric $
 */
using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;

namespace com.trackplus.track.ws
{
    /**
     * <p>
     * This class serves as a little example how to work with the Track+ client
     * library (TCL). It demonstrates how to create an issue, retrieve an issue by
     * its identifier, and change various aspects of an issue. It also
     * demonstrates how to access all issue information.</p>
     * <p>
     * The class uses the {@link LoginSample} class to perform
     * the login. You need to set the URL and the user name and password in that
     * class before you compile and run this class here.</p>
     * <p>
     * This example is a C# console application. You can run it directly by
     * calling its <code>runItemSample()</code> method.</p>
     * 
     * 
     */
    public class ItemSample : LoginSample
    {

        public ItemSample()
            : base()
        {
        }
        public ItemSample(TCLFacade connector)
            : base(connector)
        {
        }


        public String createItem()
        {
            return createItem("Item created via webservice");
        }
        /**
         * Create a new item
         * @return the new item id
         */
        public String createItem(String title)
        {
            Console.WriteLine("\n->ItemSample.createItem()");
            //to create an item we need an project and issueType
            String projectID = null;
            String issueTypeID = null;
            //getting the project for the user 
            try
            {
                WSLabelValueBean[] projects = connector.getProjects();
                if (projects == null || projects.Length == 0)
                {
                    Console.WriteLine("No projects found for user:" + user);
                    return null;
                }
                projectID = projects[0].value;
                WSLabelValueBean[] issueTypes = connector.getIssueTypes(projectID);
                if (issueTypes == null || issueTypes.Length == 0)
                {
                    Console.WriteLine("No item types found for user:" + user);
                    return null;
                }
                issueTypeID = issueTypes[0].value;
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Error obtaining the project and item type");
                handleTCLFacadeException(e);
                return null;
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            WSEditItemContextBean editItemCtx = null;
            try
            {
                editItemCtx = connector.getNewItemContext(projectID, issueTypeID);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Error obtaining the newItemContext");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            WSItemBean item = editItemCtx.item;
            //the item contains the default values
            //print item, we need the custom fields
            printEditItemContext(editItemCtx, getCustomFields());

            //the fieldsConfig contains settings(label, tooltip, readOnly) and options only for fields 
            //present on the form configured for NEW item in given project and issueType
            //and if we change a field that is not present on the form will do not have effect.
            WSFieldsConfigBean cfg = editItemCtx.fieldsConfig;

            //set some values
            item.synopsis=title;
            item.description="This item was created via the webservice";
            //set the priority( assume the priority is present on the form)
            //the priorities length must be >=1 because include the default value
            WSSortedOptionBean[] priorities = cfg.priorityOptions;
            String priorityID = priorities[priorities.Length - 1].objectID;
            item.priorityID=priorityID;
            //set the comment
            item.newComment="Comment from webservice";

            //save the item
            String itemID = null;
            try
            {
                itemID = connector.saveItem(item);
                Console.WriteLine("Item saved successfully: The new itemID is:" + itemID);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to save the item");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-ItemSample.createItem()\n");
            return itemID;
        }

        public WSItemBean getItem(String itemID)
        {
            Console.WriteLine("\n->ItemSample.item");
            WSItemBean itemBean = null;
            try
            {
                itemBean = connector.getItem(itemID);
                printItem(itemBean);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to obtain the item");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-ItemSample.item\n");
            return itemBean;
        }

        public WSDisplayItemBean getDisplayItem(String itemID)
        {
            Console.WriteLine("\n->ItemSample.getdisplayItem()");
            WSDisplayItemBean displayItem = null;
            try
            {
                displayItem = connector.getDisplayItem(itemID);
                printDisplayItem(displayItem);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to obtain the item");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-ItemSample.getdisplayItem()\n");
            return displayItem;
        }
        public WSItemContextBean getItemContext(String itemID)
        {
            Console.WriteLine("\n->ItemSample.getItemContext()");
            WSItemContextBean ctx = null;
            try
            {
                ctx = connector.getItemContext(itemID);
                printItemContext(ctx);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to obtain the item context");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-ItemSample.getItemContext()\n");
            return ctx;
        }

        public WSEditItemContextBean getEditItemContext(String itemID)
        {
            Console.WriteLine("\n->ItemSample.getEditItemContext()");
            WSEditItemContextBean editCtx = null;
            try
            {
                editCtx = connector.getEditItemContext(itemID);
                printEditItemContext(editCtx, getCustomFields());
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to obtain the item context");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-ItemSample.getEditItemContext()\n");
            return editCtx;
        }

        public void modifyItem(String itemID)
        {
            Console.WriteLine("\n->ItemSample.modifyItem()");
            WSEditItemContextBean editCtx = null;
            try
            {
                editCtx = connector.getEditItemContext(itemID);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to obtain the item context");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            if (editCtx == null)
            {
                return;
            }
            WSItemBean item = editCtx.item;
            WSFieldsConfigBean cfg = editCtx.fieldsConfig;

            //moify the description
            item.description="Description modified from webservice";

            //modify the startDate
            //DateTime startDate = new DateTime();
            String startDateISO = "2009-05-17";// TCLUtil.formatISODate(startDate);
            item.startDate=startDateISO;

            //modify the severity
           /* WSSortedOptionBean[] severites = cfg.severityOptions;
            //take the last severity
            String severityID = severites[severites.Length - 1].objectID;
            item.severityID=severityID;*/

            //modify the responsible
            WSPersonBean[] responsibleOptions = cfg.responsibleOptions;
            String responsibleID = responsibleOptions[0].personID;
            item.responsibleID=responsibleID;

            //save the item
            try
            {
                connector.saveItem(item);
                Console.WriteLine("Item modified successfully");
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to save the item");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }

            Console.WriteLine("<-ItemSample.modifyItem()\n");
        }

        /**
         * This method demonstrates how to add a comment to an issue.
         * @param itemID
         */
        public void addComment(String itemID)
        {
            Console.WriteLine("\n->ItemSample.addComment()");
            WSCommentEdit newComment = new WSCommentEdit();
            newComment.workItemID = itemID;
            newComment.description = "Some new comment via web-service";
            try
            {
                connector.editComment(newComment);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }


            WSCommentEdit[] comments = null;
            try
            {
                comments = connector.getComments(itemID);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            printComments(comments);
            Console.WriteLine("<-ItemSample.addComment()\n");
        }

        public void editComment(String itemID)
        {
            Console.WriteLine("->ItemSample.editComment()");
            WSCommentEdit[] comments = null;
            try
            {
                comments = connector.getComments(itemID);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            if (comments == null || comments.Length == 0)
            {
                Console.WriteLine("No comments found");
                return;
            }
            Console.WriteLine("Original comments");
            printComments(comments);
            WSCommentEdit comment = comments[0];
            comment.description = "Some comment modification";
            try
            {
                connector.editComment(comment);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }

            try
            {
                comments = connector.getComments(itemID);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("Modified comments");
            printComments(comments);
            Console.WriteLine("<-ItemSample.editComment()");
        }
        public void deleteComments(String itemID)
        {
            Console.WriteLine("->ItemSample.deleteComments()");
            WSCommentEdit[] comments = null;
            try
            {
                comments = connector.getComments(itemID);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            if (comments == null || comments.Length == 0)
            {
                Console.WriteLine("No comments found");
                return;
            }
            Console.WriteLine("Original comments");
            printComments(comments);
            WSCommentEdit comment = comments[0];
            String[] commentIDs = new String[] { comment.objectID };
            try
            {
                connector.deleteComments(itemID, commentIDs);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            try
            {
                comments = connector.getComments(itemID);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("modified comments");
            printComments(comments);
            Console.WriteLine("<-ItemSample.deleteComments()");
        }

        public void changeStatus(String itemID)
        {
            Console.WriteLine("\n->ItemSample.changeStatus()");
            WSEditItemContextBean editCtx = null;
            try
            {
                editCtx = connector.getChangeStatusItemContext(itemID);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to obtain the item context");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            if (editCtx == null)
            {
                return;
            }
            WSItemBean item = editCtx.item;
            WSFieldsConfigBean cfg = editCtx.fieldsConfig;

            //set the new status
            WSStateOptionBean[] states = cfg.stateOptions;
            String stateID = states[states.Length - 1].objectID;
            item.stateID = stateID;

            //set comment
            item.newComment = "Status changed from web-service";

            //save the item
            try
            {
                connector.saveItem(item);
                Console.WriteLine("Item modified successfully");
                WSEditItemContextBean editItemCtx = getEditItemContext(itemID);
                printEditItemContext(editItemCtx, getCustomFields());
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to save the item");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("->ItemSample.changeStatus()\n");
        }
        public void getHistory(String itemID)
        {
            Console.WriteLine("\n->ItemSample.getHistory()");
            WSItemHistoryBean itemHistory = null;
            try
            {
                itemHistory = connector.getItemHistory(itemID);
                printItemHistory(itemHistory);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to obtain the item context");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-ItemSample.getHistory()\n");
        }

        private void printItemHistory(WSItemHistoryBean itemHistory)
        {
            if (itemHistory == null)
            {
                Console.WriteLine("No history");
                return;
            }
            //history
            WSHistoryValues[] historyValues = itemHistory.historyValues;
            Console.WriteLine("History");
            if (historyValues == null || historyValues.Length == 0)
            {
                Console.WriteLine("\tNo history");
            }
            else
            {
                for (int i = 0; i < historyValues.Length; i++)
                {
                    Console.WriteLine("\t" + "By " + historyValues[i].changedByName + " At " + historyValues[i].lastEdit);
                    Console.WriteLine("\t" + historyValues[i].fieldName);
                    Console.WriteLine("\t new value=" + historyValues[i].newValue);
                    Console.WriteLine("\t old value=" + historyValues[i].oldValue + "\n");
                }
            }
            //The cost changes history
            WSCostBean[] costs = itemHistory.costs;
            Console.WriteLine("Cost history");
            if (costs == null || costs.Length == 0)
            {
                Console.WriteLine("\tNo cost change");
            }
            else
            {
                for (int i = 0; i < costs.Length; i++)
                {
                    Console.WriteLine("\t" + "By " + costs[i].createdByName + " At " + costs[i].lastEdit + " Account=" + costs[i].accountName);
                    Console.WriteLine("\t" + "Cost:" + costs[i].cost + " Hours:" + costs[i].hours);
                    Console.WriteLine("\t" + costs[i].description + "\n");
                }
            }
        }
        private WSField[] getCustomFields()
        {
            try
            {
                return connector.getCustomFields();
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            return new WSField[0];
        }
        private void printItem(WSItemBean item)
        {
            Console.WriteLine("\n-------WSItemBean-----------------------");

            Console.WriteLine("ChangedByID=" + item.changedByID);
           // Console.WriteLine("ClassID=" + item.classID);
            Console.WriteLine("Created=" + item.created);
            Console.WriteLine("Description=" + item.description);
            Console.WriteLine("EndDate=" + item.endDate);
            Console.WriteLine("IssueTypeID=" + item.issueTypeID);
            Console.WriteLine("LastEdit=" + item.lastEdit);
            Console.WriteLine("ManagerID=" + item.managerID);
            Console.WriteLine("NewComment=" + item.newComment);
            Console.WriteLine("ObjectID=" + item.objectID);
            Console.WriteLine("OriginatorID=" + item.originatorID);
            Console.WriteLine("PriorityID=" + item.priorityID);
            Console.WriteLine("PrivateItem=" + item.privateItem);
            Console.WriteLine("ProjectID=" + item.projectID);
            Console.WriteLine("ReleaseNoticedID=" + item.releaseNoticedID);
            Console.WriteLine("ReleaseScheduledID=" + item.releaseScheduledID);
            Console.WriteLine("ResponsibleID=" + item.responsibleID);
            Console.WriteLine("SeverityID=" + item.severityID);
            Console.WriteLine("StartDate=" + item.startDate);
            Console.WriteLine("StateID=" + item.stateID);
         //   Console.WriteLine("SubprojectID=" + item.subprojectID);
            Console.WriteLine("SuperiorWorkitem=" + item.superiorWorkitem);
            Console.WriteLine("SuperiorWorkitemSynopsys=" + item.superiorWorkitemSynopsys);
            Console.WriteLine("Synopsis=" + item.synopsis);
            Console.WriteLine();

            printCustomFields(item.customFieldValues);
            printChildrenIds(item.childrenIds);
            printAttachments(item.attachments);
            printComments(item.comments);
            printConsInf(item.consInfEdit);
            printTimeAndCost(item.timeAndCost);

            Console.WriteLine("----------------------------------------\n");

        }
        private void printDisplayItem(WSDisplayItemBean displayItem)
        {
            Console.WriteLine("\n-------WSdisplayItemBean----------------");
            WSPersonBean changedBy = displayItem.changedBy;
            Console.WriteLine(displayItem.changedByLabel+ "=" + (changedBy == null ? null : changedBy.personID + ":" + changedBy.fullName));

 //           WSOptionBean clazz = displayItem.clazz;
 //           Console.WriteLine(displayItem.clazzLabel+ "=" + (clazz == null ? null : clazz.label));

            Console.WriteLine(displayItem.createdLabel+ "=" + displayItem.created);
            Console.WriteLine(displayItem.descriptionLabel+ "=" + displayItem.description);
            Console.WriteLine(displayItem.endDateLabel+ "=" + displayItem.endDate);

            WSSortedOptionBean issueType = displayItem.issueType;
            Console.WriteLine(displayItem.issueTypeLabel+ "=" + (issueType == null ? null : issueType.label));

            Console.WriteLine(displayItem.lastEditLabel+ "=" + displayItem.lastEdit);

            WSPersonBean manager = displayItem.manager;
            Console.WriteLine(displayItem.managerLabel+ "=" + (manager == null ? null : manager.personID + ":" + manager.fullName));

            //Console.WriteLine(displayItem.NewCommentLabel()+"="+displayItem.newComment);
            Console.WriteLine(displayItem.objectIDLabel+ "=" + displayItem.objectID);

            WSPersonBean originator = displayItem.originator;
            Console.WriteLine(displayItem.originatorLabel+ "=" + (originator == null ? null : originator.personID + ":" + originator.fullName));

            WSSortedOptionBean priority = displayItem.priority;
            Console.WriteLine(displayItem.priorityLabel+ "=" + (priority == null ? null : priority.objectID + ":" + priority.label));

            Console.WriteLine("private=" + displayItem.privateItem);

            WSOptionBean project = displayItem.project;
            Console.WriteLine(displayItem.projectLabel+ "=" + (project == null ? null : project.objectID + ":" + project.label));

            WSSortedOptionBean releaseNoticed = displayItem.releaseNoticed;
            Console.WriteLine(displayItem.releaseNoticedLabel+ "=" + (releaseNoticed == null ? null : releaseNoticed.objectID + ":" + releaseNoticed.label));

            WSSortedOptionBean releaseScheduled = displayItem.releaseScheduled;
            Console.WriteLine(displayItem.releaseScheduledLabel+ "=" + (releaseScheduled == null ? null : releaseScheduled.objectID + ":" + releaseScheduled.label));

            WSPersonBean responsible = displayItem.responsible;
            Console.WriteLine(displayItem.responsibleLabel+ "=" + (responsible == null ? null : responsible.personID + ":" + responsible.fullName));

            WSSortedOptionBean severity = displayItem.severity;
            Console.WriteLine(displayItem.severityLabel+ "=" + (severity == null ? null : severity.objectID + ":" + severity.label));

            Console.WriteLine(displayItem.startDateLabel+ "=" + displayItem.startDate);

            WSStateOptionBean state = displayItem.state;
            Console.WriteLine(displayItem.stateLabel+ "=" + (state == null ? null : state.objectID + ":" + state.label));

//            WSOptionBean subproject = displayItem.subproject;
//            Console.WriteLine(displayItem.subprojectLabel+ "=" + (subproject == null ? null : subproject.objectID + ":" + subproject.label));

            Console.WriteLine(displayItem.superiorWorkitemLabel+ "=" + displayItem.superiorWorkitem);
            Console.WriteLine("Synopsis=" + displayItem.superiorWorkitemSynopsys);
            Console.WriteLine(displayItem.synopsisLabel+ "=" + displayItem.synopsis);
            Console.WriteLine();

            printCustomDisplayFields(displayItem.customFieldValues);
            pritCustomFieldsLabels(displayItem.customFieldLabels);
            printChildrenIds(displayItem.childrenIds);
            printAttachments(displayItem.attachments);
            printComments(displayItem.comments);
            printConsInf(displayItem.consInfShow);
            printTimeAndCost(displayItem.timeAndCost);

            Console.WriteLine("----------------------------------------\n");
        }

        public void printEditItemContext(WSEditItemContextBean editCtx, WSField[] customFields) {
		Console.WriteLine("\n----------------------------------------");
		//ATTENTION the setting and options are present ONLY for fields present on form 
		//WSFieldsConfigBean cfg=editCtx.fieldsConfig;
		WSItemBean item=editCtx.item;
		//create an wrapper over editItemContext
		ItemWrapper itemWrapper=new ItemWrapper(customFields,editCtx);
		//FormSample.printForm(editCtx.getForm);
		ICollection<int> presentFields=itemWrapper.getPresentFields();
        IEnumerator<int> it = presentFields.GetEnumerator();
		while(it.MoveNext()){
			String fieldID=it.Current+"";
			Console.Write("FieldID="+fieldID);
			//Console.Write("\tType="+itemWrapper.get);
			Console.Write(" Label="+itemWrapper.getLabel(fieldID));
			Console.Write(" Value="+itemWrapper.getValue(fieldID));
			Console.Write(" Display Value="+itemWrapper.getDisplayValue(fieldID));
			Console.Write(" ReadOnly="+itemWrapper.isReadOnly(fieldID));
			Console.WriteLine();
		}
		
		printCustomFields(item.customFieldValues);
		printChildrenIds(item.childrenIds);
		printAttachments(item.attachments);
		printComments(item.comments);
		printConsInf(item.consInfEdit);
		printTimeAndCost(item.timeAndCost);
		
		Console.WriteLine("----------------------------------------\n");
	}

        private void printItemContext(WSItemContextBean ctx)
        {
            WSItemBean item = ctx.item;
            WSFieldsConfigBean cfg = ctx.fieldsConfig;
            Console.WriteLine("\n\n----------------------------------------");
            Console.WriteLine(cfg.changedBySettings.label+ "=" + TCLUtil.findDisplayValue(cfg.changedByOptions, item.changedByID));
            //Console.WriteLine(cfg.classSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.classOptions, item.classID));
            Console.WriteLine(cfg.createdSettings.label+ "=" + item.created);
            Console.WriteLine(cfg.descriptionSettings.label+ "=" + item.description);
            Console.WriteLine(cfg.endDateSettings.label+ "=" + item.endDate);
            Console.WriteLine(cfg.issueTypeSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.issueTypeOptions, item.issueTypeID));
            Console.WriteLine(cfg.lastEditSettings.label+ "=" + item.lastEdit);
            Console.WriteLine(cfg.managerSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.managerOptions, item.managerID));
            Console.WriteLine(cfg.commentSettings.label+ "=" + item.newComment);
            Console.WriteLine(cfg.issueNoSettings.label+ "=" + item.objectID);
            Console.WriteLine(cfg.originatorSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.originatorOptions, item.originatorID));
            Console.WriteLine(cfg.prioritySettings.label+ "=" + TCLUtil.findDisplayValue(cfg.priorityOptions, item.priorityID));
            //no localization label for private from WSFieldsConfigBean
            Console.WriteLine("Private=" + item.privateItem);
            Console.WriteLine(cfg.projectSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.projectOptions, item.projectID));
            Console.WriteLine(cfg.releaseNoticedSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.releaseNoticedOptions, item.releaseNoticedID));
            Console.WriteLine(cfg.releaseScheduledSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.releaseScheduledOptions, item.releaseScheduledID));
            Console.WriteLine(cfg.responsibleSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.responsibleOptions, item.responsibleID));
            //Console.WriteLine(cfg.severitySettings.label+ "=" + TCLUtil.findDisplayValue(cfg.severityOptions, item.severityID));
            Console.WriteLine(cfg.startDateSettings.label+ "=" + item.startDate);
            Console.WriteLine(cfg.stateSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.stateOptions, item.stateID));
            //Console.WriteLine(cfg.subprojectSettings.label+ "=" + TCLUtil.findDisplayValue(cfg.subprojectOptions, item.subprojectID));
            Console.WriteLine(cfg.superiorWorkitemSettings.label+ "=" + item.superiorWorkitem+ " " + item.superiorWorkitemSynopsys);
            Console.WriteLine(cfg.synopsysSettings.label+ "=" + item.synopsis);
            Console.WriteLine();

            printCustomFields(item.customFieldValues);
            printChildrenIds(item.childrenIds);
            printAttachments(item.attachments);
            printComments(item.comments);
            printConsInf(item.consInfEdit);
            printTimeAndCost(item.timeAndCost);

            Console.WriteLine("----------------------------------------\n\n");
        }

        private void printTimeAndCost(WSTimeAndCost timeAndCost)
        {
            if (timeAndCost == null)
            {
                Console.WriteLine("No time and cost");
            }
            else
            {
                Console.WriteLine("Time and cost:");
                String currency = timeAndCost.currency;
                Console.WriteLine("\tCurrency:" + currency);
                String hoursperWorkDay = timeAndCost.hoursPerWorkDay;
                Console.WriteLine("\tHours per work day=" + hoursperWorkDay);
                WSAccountBean[] accounts = timeAndCost.acounts;
                if (accounts == null || accounts.Length == 0)
                {
                    Console.WriteLine("\tNo accounts:");
                }
                else
                {
                    Console.WriteLine("\tAccounts:");
                    for (int i = 0; i < accounts.Length; i++)
                    {
                        Console.WriteLine("\t\t" + accounts[i].objectID + " " + accounts[i].fullName);
                    }
                }
                WSBudgetBean budgetBean = timeAndCost.budget;
                if (budgetBean == null)
                {
                    Console.WriteLine("\tNo budget");
                }
                else
                {
                    Console.WriteLine("\tBudget");
                    Console.WriteLine("\t\tChangedByID=" + budgetBean.changedByID);
                    Console.WriteLine("\t\tChangedByName=" + budgetBean.changedByName);
                    Console.WriteLine("\t\tDescription=" + budgetBean.description);
                    Console.WriteLine("\t\tLastEdit=" + budgetBean.lastEdit);
                    Console.WriteLine("\t\tObjectID=" + budgetBean.objectID);
                    Console.WriteLine("\t\tTimeUnit=" + budgetBean.timeUnit);
                    Console.WriteLine("\t\tTotalCost=" + budgetBean.totalCost);
                    Console.WriteLine("\t\tTotalHours=" + budgetBean.totalHours);
                    //Console.WriteLine("\t\tWorkItemID="+budgetBean.getWorkItemID);
                }
                WSEstimatedBudgetBean estimatedBudgetBean = timeAndCost.estimatedBudget;
                if (estimatedBudgetBean == null)
                {
                    Console.WriteLine("\tNo Estimated Budget");
                }
                else
                {
                    Console.WriteLine("\tEstimated Budget");
                    Console.WriteLine("\t\tChangedByID=" + estimatedBudgetBean.changedByID);
                    Console.WriteLine("\t\tChangedByName=" + estimatedBudgetBean.changedByName);
                    Console.WriteLine("\t\tLastEdit=" + estimatedBudgetBean.lastEdit);
                    Console.WriteLine("\t\tObjectID=" + estimatedBudgetBean.objectID);
                    Console.WriteLine("\t\tTimeUnit=" + estimatedBudgetBean.timeUnit);
                    Console.WriteLine("\t\tEstimatedCost=" + estimatedBudgetBean.estimatedCost);
                    Console.WriteLine("\t\tEstimatedHours=" + estimatedBudgetBean.estimatedHours);
                    //Console.WriteLine("\t\tWorkItemID="+budgetBean.getWorkItemID);
                }
                WSCostBean[] costs = timeAndCost.costs;
                if (costs == null || costs.Length == 0)
                {
                    Console.WriteLine("\tNo costs");
                }
                else
                {
                    Console.WriteLine("\tCosts:");
                    for (int i = 0; i < costs.Length; i++)
                    {
                        Console.WriteLine("\t\t" + costs[i].objectID +
                                " Account=" + costs[i].accountName+
                                " Cost=" + costs[i].cost+
                                " Time=" + costs[i].hours+
                                " Effortdate=" + costs[i].effortdate+
                                " By " + costs[i].createdByName+
                                " At " + costs[i].lastEdit);
                    }
                }
            }
        }
        private void printChildrenIds(String[] childrenIds) {
		if(childrenIds==null||childrenIds.Length==0){
			Console.WriteLine("No children");
		}else{
			Console.Write("Children:");
			for (int i = 0; i < childrenIds.Length; i++) {
				Console.Write(childrenIds[i]+";");
			}
			Console.WriteLine("\n");
		}
	}
        private void printCustomFields(WSCustomFieldValue[] customFields)
        {
            if (customFields == null || customFields.Length == 0)
            {
                Console.WriteLine("No custom fields");
            }
            else
            {
                Console.WriteLine("Custom fields:");
                for (int i = 0; i < customFields.Length; i++)
                {
                    Console.WriteLine("\t" + customFields[i].fieldId+ "=" + customFields[i].value);
                }
            }
        }
        private void printCustomDisplayFields(WSCustomFieldDisplayValue[] customFieldValues)
        {
            if (customFieldValues == null || customFieldValues.Length == 0)
            {
                Console.WriteLine("No custom fields");//
            }
            else
            {
                Console.WriteLine("Custom fields:");
                for (int i = 0; i < customFieldValues.Length; i++)
                {
                    Console.WriteLine("\t" + customFieldValues[i].fieldId+
                             " label=" + "value=" + customFieldValues[i].value + ":" + customFieldValues[i].displayValue);
                }
            }
        }
        private void pritCustomFieldsLabels(WSLabelValueBean[] customFieldLabels)
        {
            if (customFieldLabels == null || customFieldLabels.Length == 0)
            {
                Console.WriteLine("No custom fields");//
            }
            else
            {
                Console.WriteLine("Custom fields labels:");
                for (int i = 0; i < customFieldLabels.Length; i++)
                {
                    Console.WriteLine("\tFieldID=" + customFieldLabels[i].value + " Lable=" + customFieldLabels[i].label);
                }
            }

        }
        private void printConsInf(WSConsInfEdit consInfEdit)
        {
            WSPersonBean[] consGroups = consInfEdit.consultantGroups;
            WSPersonBean[] consPersons = consInfEdit.consultantPersons;
            WSLabelValueBean[] selectedConsGroups = consInfEdit.selectedConsultantGroups;
            WSLabelValueBean[] selectedConsPersons = consInfEdit.selectedConsultantPersons;
            Console.WriteLine("Consulted");
            if (consGroups == null || consGroups.Length == 0)
            {
                Console.WriteLine("\tNo possible groups:");
            }
            else
            {
                Console.WriteLine("\tPossible groups:");
                for (int i = 0; i < consGroups.Length; i++)
                {
                    Console.WriteLine("\t\t" + consGroups[i].personID + ":" + consGroups[i].fullName);
                }
            }
            if (consPersons == null || consPersons.Length == 0)
            {
                Console.WriteLine("\tNo possible persons:");
            }
            else
            {
                Console.WriteLine("\tPossible persons:");
                for (int i = 0; i < consPersons.Length; i++)
                {
                    Console.WriteLine("\t\t" + consPersons[i].personID + ":" + consPersons[i].fullName);
                }
            }
            if (selectedConsGroups == null || selectedConsGroups.Length == 0)
            {
                Console.WriteLine("\tNo Selected groups:");
            }
            else
            {
                Console.WriteLine("\tSelected groups:");
                for (int i = 0; i < selectedConsGroups.Length; i++)
                {
                    Console.WriteLine("\t\t" + selectedConsGroups[i].value + ":" + selectedConsGroups[i].label);
                }
            }
            if (selectedConsPersons == null || selectedConsPersons.Length == 0)
            {
                Console.WriteLine("\tNo selected persons:");
            }
            else
            {
                Console.WriteLine("\tSelected persons:");
                for (int i = 0; i < selectedConsPersons.Length; i++)
                {
                    Console.WriteLine("\t\t" + selectedConsPersons[i].value + ":" + selectedConsPersons[i].label);
                }
            }
            WSPersonBean[] infGroups = consInfEdit.informantGroups;
            WSPersonBean[] infPersons = consInfEdit.informantPersons;
            WSLabelValueBean[] selectedInfGroups = consInfEdit.selectedInformantGroups;
            WSLabelValueBean[] selectedInfPersons = consInfEdit.selectedInformantPersons;
            Console.WriteLine("Informants");
            if (infGroups == null || infGroups.Length == 0)
            {
                Console.WriteLine("\tNo possible groups:");
            }
            else
            {
                Console.WriteLine("\tPossible groups:");
                for (int i = 0; i < infGroups.Length; i++)
                {
                    Console.WriteLine("\t\t" + infGroups[i].personID + ":" + infGroups[i].fullName);
                }
            }
            if (infPersons == null || infPersons.Length == 0)
            {
                Console.WriteLine("\tNo possible persons:");
            }
            else
            {
                Console.WriteLine("\tPossible persons:");
                for (int i = 0; i < infPersons.Length; i++)
                {
                    Console.WriteLine("\t\t" + infPersons[i].personID + ":" + infPersons[i].fullName);
                }
            }
            if (selectedInfGroups == null || selectedInfGroups.Length == 0)
            {
                Console.WriteLine("\tNo Selected groups:");
            }
            else
            {
                Console.WriteLine("\tSelected groups:");
                for (int i = 0; i < selectedInfGroups.Length; i++)
                {
                    Console.WriteLine("\t\t" + selectedInfGroups[i].value + ":" + selectedInfGroups[i].label);
                }
            }
            if (selectedInfPersons == null || selectedInfPersons.Length == 0)
            {
                Console.WriteLine("\tNo Selected persons:");
            }
            else
            {
                Console.WriteLine("\tSelected persons:");
                for (int i = 0; i < selectedInfPersons.Length; i++)
                {
                    Console.WriteLine("\t\t" + selectedInfPersons[i].value + ":" + selectedInfPersons[i].label);
                }
            }
        }
        private void printConsInf(WSConsInfShow consInfShow)
        {
            WSPersonBean[] consGroups = consInfShow.consultantGroups;
            WSPersonBean[] consPersons = consInfShow.consultantPersons;
            Console.WriteLine("Consulted");
            if (consGroups == null || consGroups.Length == 0)
            {
                Console.WriteLine("\tNo groups:");
            }
            else
            {
                Console.WriteLine("\tGroups:");
                for (int i = 0; i < consGroups.Length; i++)
                {
                    Console.WriteLine("\t\t" + consGroups[i].personID + ":" + consGroups[i].fullName);
                }
            }
            if (consPersons == null || consPersons.Length == 0)
            {
                Console.WriteLine("\tNo persons:");
            }
            else
            {
                Console.WriteLine("\tPersons:");
                for (int i = 0; i < consPersons.Length; i++)
                {
                    Console.WriteLine("\t\t" + consPersons[i].personID + ":" + consPersons[i].fullName);
                }
            }
            WSPersonBean[] infGroups = consInfShow.informantGroups;
            WSPersonBean[] infPersons = consInfShow.informantPersons;
            Console.WriteLine("Informed");
            if (infGroups == null || infGroups.Length == 0)
            {
                Console.WriteLine("\tNo groups:");
            }
            else
            {
                Console.WriteLine("\tGroups:");
                for (int i = 0; i < infGroups.Length; i++)
                {
                    Console.WriteLine("\t\t" + infGroups[i].personID + ":" + infGroups[i].fullName);
                }
            }
            if (infPersons == null || infPersons.Length == 0)
            {
                Console.WriteLine("\tNo persons:");
            }
            else
            {
                Console.WriteLine("\tPersons:");
                for (int i = 0; i < infPersons.Length; i++)
                {
                    Console.WriteLine("\t\t" + infPersons[i].personID + ":" + infPersons[i].fullName);
                }
            }
        }
        private void printAttachments(WSAttachmentBean[] attchments)
        {
            if (attchments == null || attchments.Length == 0)
            {
                Console.WriteLine("No attachments");
            }
            else
            {
                Console.WriteLine("Attachments: " + attchments.Length);
                for (int i = 0; i < attchments.Length; i++)
                {
                    Console.WriteLine("\t" + attchments[i].attachKey+ ":" + attchments[i].attachName+ " " + attchments[i].sizeKb+ "Kb" + " By " + attchments[i].originator.fullName);
                }
            }
        }
        private void printComments(WSComment[] comments)
        {
            if (comments == null || comments.Length == 0)
            {
                Console.WriteLine("No comments");
            }
            else
            {
                Console.WriteLine("Comments: " + comments.Length);
                for (int i = 0; i < comments.Length; i++)
                {
                    Console.WriteLine("\t By " + comments[i].originator.fullName+ " At" + comments[i].createdDate);
                }
                Console.WriteLine();
            }
        }
        private void printComments(WSCommentEdit[] comments)
        {
            if (comments == null || comments.Length == 0)
            {
                Console.WriteLine("No comments");
            }
            else
            {
                Console.WriteLine("Comments: " + comments.Length);
                for (int i = 0; i < comments.Length; i++)
                {
                    Console.WriteLine("\t "+comments[i].objectID+" By " + comments[i].originator.fullName + " At" + comments[i].createdDate);
                    Console.WriteLine(comments[i].description+"\n");
                }
                Console.WriteLine();
            }
        }

        public static void runItemSample(String[] args)
        {
            ItemSample itemSample = new ItemSample();
            //login
            itemSample.login(user, passwd);

            String itemID = itemSample.createItem();
            if (itemID != null)
            {
                itemSample.getItem(itemID);
                itemSample.getDisplayItem(itemID);
                itemSample.getItemContext(itemID);
                itemSample.getEditItemContext(itemID);
                itemSample.modifyItem(itemID);
                itemSample.addComment(itemID);
                itemSample.editComment(itemID);
                itemSample.deleteComments(itemID);
                itemSample.changeStatus(itemID);
                itemSample.getHistory(itemID);
            }
        }
    }
}
