/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2014 Steinbeis GmbH & Co. KG
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id: LoginSample.cs 1465 2015-10-01 11:56:46Z friedric $
 */
using System;
using System.Collections.Generic;
using System.Text;
using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;
using log4net;
using log4net.Config;


namespace com.trackplus.track.ws
{
    /**
     * <p>
     * This class demonstrates how to use the TCl library to login into a Track+ server
     * via the web service interface. You need to replace the URL below and have it point
     * to your server. Furthermore you need to set the user login name and
     * password.</p>
     * <p>
     * This example is a C# console application. You can run it directly by
     * calling its <code>runLoginSample()</code> method.</p>
     *
     */
    public class LoginSample
    {
        private static readonly ILog LOGGER = LogManager.GetLogger(typeof(LoginSample));

        // public const String URL = "http://192.168.178.74:8080/track/services/TrackplusService";
        // public const String URL = "https://www.trackplus.com/track-demo/services/TrackplusService";
        public const String URL = "http://www.trackplus.com/track-demo/services/TrackplusService";
        // public const String URL = "http://192.168.0.103:8080/track/services/TrackplusService";
        /**
         * The user login name
         */
        protected static String user = "sysman";
        /**
         * The user password
         */
        protected static String passwd = "demos3%";

        /**
         * The client facade over web-service 
         */
        protected TCLFacade connector;

        /**
         * Constructor
         */
        public LoginSample()
        {
           
            try
            {
                connector = new TCLFacade(URL);
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }

        public LoginSample(TCLFacade connector)
        {
            this.connector = connector;
        }
        public void login()
        {
            login(user, passwd);
        }
        /**
         * Login via web-service in track+ system
         * @param userlogin
         * @param password
         */
        public void login(String userlogin, String password)
        {
            Console.WriteLine("\n->LoginSample.login("+userlogin+","+password+")");
            //create login information
            WSLogonBean logonBean = new WSLogonBean();
            logonBean.username = userlogin;
            logonBean.password = password;

            /*
             * Setting the client locale
             * If on the server there is no preferred locale for this user 
             * we use this locale.
            */
            logonBean.locale = "en_EN";
            try
            {
                WSPersonBean personBean = connector.login(logonBean);
                if (personBean != null)
                {
                    Console.WriteLine("Logged in successfully as: " + personBean.fullName);
                    Console.WriteLine("Person preferred locale: " + personBean.preferredLocale);
                }
                else
                {
                    Console.WriteLine("Login refused.");
                }
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message: " + e.Message);
            }

            Console.WriteLine("<-LoginSample.login()\n");
        }

        /**
         * Check the server version and print the compatible versions
         */
        public void checkVersion()
        {
            Console.WriteLine("\n->LoginSample.checkVersion()");
            try
            {
                bool b = connector.checkVersion();
                if (b)
                {
                    Console.WriteLine("Client and server versions are compatible!");
                }
                else
                {
                    Console.WriteLine("INCOMPATIBLE client and server versions!!!");
                }
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message: " + e.Message);
            }

            //getting the server compatible versions

            //the first element in the array is the server version 
            //and the rest are the client compatible versions
            try
            {
                String[] serverVersionAndCopatibility = connector.getServerVersionAndCompatible();
                Console.WriteLine("Server version:" + serverVersionAndCopatibility[0]);
                Console.Write("Client compatible version are:");
                for (int i = 1; i < serverVersionAndCopatibility.Length; i++)
                {
                    Console.Write(serverVersionAndCopatibility[i] + ";");
                }
                Console.WriteLine();
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message: " + e.Message);
            }
            Console.WriteLine("<-LoginSample.checkVersion()\n");
        }

        /**
         * Simple Handler for  TCLFacadeException
         * @param e
         */
        protected void handleTCLFacadeException(TCLFacadeException e)
        {
            //handling the messages
            Console.WriteLine("TCLFacadeException.getErrorCode()=" + e.getErrorCode());
            //this messages are localized from server
            String[] errorMessages = e.getErrorMessages();
            if (errorMessages != null && errorMessages.Length > 0)
            {
                for (int i = 0; i < errorMessages.Length; i++)
                {
                    Console.WriteLine("TCLFacadeException.ErrorMessage[" + i + "]=" + errorMessages[i]);
                }
            }
            Console.WriteLine("TCLFacadeException.message:" + e.Message);
            //e.printStackTrace();
        }


        public static void runLoginSample(String[] args)
        {
            LoginSample loginSample = new LoginSample();
            loginSample.login(user, passwd);

            //Incorrect login
            loginSample.login(user, null);

            //Check version
            loginSample.checkVersion();
        }

    }
}
