/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2014 Steinbeis GmbH & Co. KG
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id: QuerySample.cs 1344 2015-09-07 20:39:21Z friedric $
 */
using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;

namespace com.trackplus.track.ws
{
    /**
     * <p>
     * This class demonstrates how to retrieve a number of issues from a Track+
     * server by a query via the web service interface using the Track+ Client
     * Library (TCL). The class uses the {@link LoginSample} class to perform
     * the login. You need to set the URL and the user name and password in that
     * class before you compile and run this class here.</p>
     * <p>
     * This example is a C# console application. You can run it directly by
     * calling its <code>runQuerySample()</code> method.</p>
     *
     */
    public class QuerySample : LoginSample
    {
        public QuerySample()
            : base()
        {

        }
        public QuerySample(TCLFacade connector)
            : base(connector)
        {
        }

        public WSTreeNode[] getQueries()
        {
            Console.WriteLine("\n->QuerySample.getQueries()");
            WSTreeNode[] result = null;
            try
            {
                result = connector.getQueries();
                printQueries(result);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-QuerySample.getQueries()\n");
            return result;
        }
        private void printQueries(WSTreeNode[] nodes)
        {
            if (nodes != null)
            {
                for (int i = 0; i < nodes.Length; i++)
                {
                    BaseOptionSample.printNode(nodes[i], "");
                }
            }
        }

        public void executeQuery(String queryID)
        {
            try
            {
                WSQueryResultItemBean[] reportBeans = connector.executeQuery(queryID);
                printReportBeans(reportBeans);

            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
        }

        public void executeCustomQuery()
        {
            Console.WriteLine("\n->QuerySample.executeCustomQuery()");
            WSQueryConfigBean reportConfigBean = new WSQueryConfigBean();
            try
            {
                reportConfigBean = connector.loadQueryConfigBean(reportConfigBean, true);
                //set states 1 (opened);
                reportConfigBean.selectedStates = new String[] { "1" };
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            try
            {
                WSQueryResultItemBean[] reportBeans = connector.executeCustomQuery(reportConfigBean);
                printReportBeans(reportBeans);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-QuerySample.executeCustomQuery()\n");
        }

        private void printReportBeans(WSQueryResultItemBean[] reportBeans)
        {
            if (reportBeans == null || reportBeans.Length == 0)
            {
                Console.WriteLine("No issues found.");
            }
            else
            {
                int printRows = reportBeans.Length;
                Console.WriteLine(printRows + " issues founds!");
                if (printRows > 10)
                {
                    printRows = 10;
                    Console.WriteLine("The first 10 issues:");
                }
                Console.WriteLine("ItemNo: Project: Subsystem: Title: Status: Responsible: LastModified");
                for (int i = 0; i < printRows; i++)
                {
                    WSQueryResultItemBean row = reportBeans[i];
                    Console.Write(row.objectID + ": ");
                    Console.Write(row.project.label + ": ");
                    //System.out.print(row.getSubproject().getLabel()+": ");
                    Console.Write(row.synopsis + ": ");
                    Console.Write(row.responsible.fullName + ": ");
                    Console.Write(row.lastEdit + "\n");
                }
            }
        }
        public void loadReportConfigBean()
        {
            Console.WriteLine("\n->QuerySample.loadReportConfigBean()");
            WSQueryConfigBean reportConfigBean = new WSQueryConfigBean();
            try
            {
                reportConfigBean = connector.loadQueryConfigBean(reportConfigBean, true);
                Console.WriteLine("Projects:" + reportConfigBean.projects.Length);
                Console.WriteLine("Issue types:" + reportConfigBean.issueTypes.Length);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-QuerySample.loadReportConfigBean()\n");
        }
        public static void runQuerySample(String[] args)
        {
            QuerySample querySample = new QuerySample();
            querySample.login();

            WSTreeNode[] queries = querySample.getQueries();
            if (queries != null)
            {
                String queryID = null;

                if (queryID != null)
                {
                    querySample.executeQuery(queryID);
                }
            }

            querySample.loadReportConfigBean();

            querySample.executeCustomQuery();


        }
    }
}

